; MSU1 ASM by Conn, yoshifanatic 07/26/2015, improved and fully optimized by NTI Productions.
; Works on unheadered (.sfc) roms, assembler with asar.

!True		= 1
!False		= 0

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;=======================================================================================================================;;
;;= Configuration == Configuration == Configuration == Configuration == Configuration == Configuration == Configuration =;;
;;=======================================================================================================================;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;


!LuigiSFX	= !True					; !True = Activate alternative SFX songs for Player 2, !False = No alternative SFX songs for both players
!OWMusic	= !True					; !True = Activate OW Music patch by smkdan, !False = Deactivate multiple songs on Overworld

!FreeRAM1	= $1E00|!Base2				;\
!FreeRAM2	= $1DEF|!Base2				;|
!FreeRAM3	= $140A|!Base2				;| Set these to unused RAM addresses if already in use
!FreeRAM4	= $140B|!Base2				;|
!FreeRAM5	= $140C|!Base2				;/

!BusyFlag	= $1DFD|!Base2				; Free RAM address for the MSU-1 pending flag, change it if already in use too
!Pausebright	= $06
!Unpausebright	= $0F

!Volume		= $FF					;/ MSU-1 volume, touching this is unneeded


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;	SA-1 compatibility check and conditionals									;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;


if read1($00FFD5) == $23
!SA1 = !True
sa1rom
else
!SA1 = !False
lorom
endif

if !SA1
!Base1 = $3000
!Base2 = $6000
else
!Base1 = $0000
!Base2 = $0000
endif


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;	Macros for easying the pain of writing the same code several times xp						;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;


macro CheckMSU(labelToJump)
	LDA $2002
	CMP #$53
	BEQ <labelToJump>
endmacro

macro CheckNoMSU(labelToJump)
	LDA $2002
	CMP #$53
	BNE <labelToJump>
endmacro


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;	Wait Replace patch by p4plus2 to increase emulator frame-rate							;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;


if !SA1
	pushpc
org $008075
	autoclean JML MSU1
else
org $008069
	BRA Skip

Back:
	WAI
	LDA $10
	BEQ Back

Skip:
	CLI
	INC $13
	JSR $9322
	JMP Foward

	pushpc
org $00FFF0
Foward:
	autoclean JML MSU1				; Hijack the main game loop and jump to the MSU-1 routine
endif


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;	MSU-1 hijacks													;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;


org $00810E
	autoclean JSL FadeOut
	NOP

org $008186
	autoclean JML MuteSPC
	NOP #5

org $009E17
	autoclean JSL FadeMusic
	NOP

org $00A23B
	autoclean JSL Paused

org $0491E0
	autoclean JSL FadeMusic
	NOP
	pullpc

freecode
print "Inserted at $",pc
print "Patch Size: ", freespaceuse, " bytes."


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;	Restore music from the SPC engine for non-MSU emulators								;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;


MuteSPC:
	PHP
	REP #$30
	PHA
	SEP #$30

	%CheckMSU(Nop)

	LDA $1DFB|!Base2				;\
	STA $2142					;| Restore noped code
	STA $1DFF|!Base2				;|
	STZ $1DFB|!Base2				;/

Nop:
	REP #$30
	PLA
	PLP
	JML $00818F


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;	Beginning and Ending of the patch, in short									;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;


MSU1:
	PHP
	REP #$30
	PHA
	PHX
	PHY
	PHD
	PHB
	PHK
	PLB
	SEP #$30

	%CheckMSU(MSUFound)

NoMSU:
	REP #$30
	PLB
	PLD
	PLY
	PLX
	PLA
	PLP
	STZ $10						; Otherwise, restore the overwritten SMW code and jump back to the start of the main loop

if !SA1
	JML $00806B
else
	JML Back
endif


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;	MSU-1 code which plays high quality music									;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;


MSUFound:

if !OWMusic
	LDA !FreeRAM4					;\ See if character is on Main Overworld so the OW Music routine can be handled
	BEQ Run						;/
	CMP #$FF					;\ Check if music has changed on Main map
	BEQ Update					;/

	LDA !BusyFlag					;\ Wait for the next frame when this flag is set
	BNE MSUReady					;/

	LDY $0DB3|!Base2				;\ Mario or Luigi?
	BEQ Mario					;/
	LDA $1F12|!Base2				;\ If Luigi is on Main map, don't branch
	BNE DontRun					;/
	JSR GetMusicIndexL				; Get Luigi position on Overworld
	BRA KeepPlay

Mario:
	LDA $1F11|!Base2				;\ If Mario is on Main map, don't branch
	BNE DontRun					;/
	JSR GetMusicIndexM				; Get Mario position on Overworld

KeepPlay:
	LDA.l MusicTBL,x				;\ Load music index from table to compare them to the value in free RAM, if the same, don't do anything.
	CMP !FreeRAM4					;| If it has changed, don't branch so the music register can be updated with the new song
	BEQ Run						;/
	LDA $13D9|!Base2				;\
	CMP #$0A					;| Fix bug which happens during switch between submap->overworld and vice-versa
	BEQ NoMSU					;/

NextFrame:
	LDA #$FF					;\
	STA !FreeRAM4					;| Set the flag to change the music
	BRA NoMSU					;/

DontRun:
	STZ !FreeRAM4					; Set submap flag, this prevents unnecessary routines to be runned
	BRA NoMSU
endif

Run:
	LDA $1DFB|!Base2				;\ This prevents the MSU-1 routine be handled again after the music is settled
	BEQ NoMSU					;/
	CMP #$80
	BEQ THEFade

if !OWMusic
	STA !FreeRAM5					; Store music to free RAM for restoring of the SPC fallback functionality with OW Music patch
endif

Update:
	LDA !BusyFlag
	BNE MSUReady

	LDA #$80					;\
	STA $2142					;| Mute SPC engine
	STA $1DFF|!Base2				;/

if !OWMusic
	STZ !FreeRAM4
endif

	JMP BankSwitch

Continue1:
	LDA $1DFB|!Base2				; Load hex track #$XX from SPC engine to be replaced by MSU decimal equivalent

Continue2:
if !LuigiSFX
	JSR LuigiPlay					; Do Luigi subroutine for separate SFX tracks, this will only assembler into the rom if !LuigiSFX = !True
endif

Continue3:
	STA !FreeRAM1					; Store song value to free RAM for future use
	STA $2004					; Load MSU track
	STZ $2005

	LDA #$01					;\ Set busy flag so the MSU-1 gets his properties on next frame
	STA !BusyFlag					;/
	JMP NoMSU

MSUReady:
	BIT $2000					;\ If not ready, loop
	BVS Mute					;/
	STZ !BusyFlag					; Clear busy flag

	LDX !FreeRAM1					;\ 
	LDA TrackLoop,x					;| Rescue song value from free RAM so setting the loop for each track be possible
	STA $2007					;/
	STA !FreeRAM2					; Store $2007 value in FreeRAM for pause
	LDA #!Volume
	STA $2006
	LDA $2000					;\
	AND #$08					;| Check error bit for SPC fallback (only featured bsnes 0.8 and higher as well as sdnes)
	BNE PlaySPC					;| and then do not allow the SPC engine to play one more time
	STZ $1DFB|!Base2				;/
	JMP NoMSU


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;	Music Fade Out													;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;


THEFade:
	LDA !FreeRAM3
	BEQ Mute

	SEC						;\
	SBC #$08					;| Fade out speed, increase to make it quickly or decrease to slow it down
	CMP #$10					;|
	BCC Go						;/

	STA !FreeRAM3					;\ Update Fade registers every frame
	STA $2006					;/
	BRA Mute

Go:
	STZ !FreeRAM3					;\
							;|
if !OWMusic						;|
	STZ !FreeRAM4					;| Finish the Fade Out and clear some flags
endif							;|
							;|
	STZ $2007					;|
	STZ $1DFB|!Base2				;/

Mute:
	JMP NoMSU


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;	SPC Fallback													;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;


PlaySPC:
	STZ $2007

if !OWMusic
	STZ !FreeRAM4
	LDA !FreeRAM5
	CMP #$02
	BEQ OWSPC
endif

SPC:
	LDA $1DFB|!Base2

OWSPC:
	STA $2142					;\ If MSU-1 track is missing, make sure to stop MSU-1 audio and restore the SPC music
	STA $1DFF|!Base2				;| from the game
	STZ $1DFB|!Base2				;/
	JMP NoMSU


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;	Luigi's new MSU-1 number set, this will assembler only if !LuigiSFX = !True					;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;


if !LuigiSFX
LuigiPlay:
	PHA
	PHX
	STA !FreeRAM1
	LDY $0DB3|!Base2				;\ Check if player is Luigi for the necessary changes to the music register
	BEQ NoAlt					;/

	LDX !FreeRAM1					;\
	LDA Luigi,x					;| Get value from Luigi table to set what songs will loop
	BNE NewMSUSet					;/

NoAlt:
	PLX
	PLA
	BRA AddL

NewMSUSet:
	PLX
	PLA
	CLC						;\
	ADC #$21					;/ Luigi number set starts +21 hex after Mario Game Over SFX (Luigi Game Over = $2A and so on)

AddL:
	RTS

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;	Choose what tracks will play different if Luigi is playing. Values: $00 - Don't change, $01 - Yes, change	;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;


Luigi:
	db $00,$00,$00,$00,$00,$00,$00,$00,$00,$01,$01,$01,$01,$01,$00,$00	;\ 00-0F
	db $01,$01,$00,$00,$01,$00,$00,$00,$00,$00,$00,$01,$01,$00,$00,$00	;| 10-1F
	db $00,$00,$00,$00,$00,$00,$00,$00,$00,$01,$00,$00,$00,$00,$00,$00	;| 20-29
	db $00,$00,$00,$00,$00							;/ 30-34
endif


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;	Select the hex track to loop, possible values are: $00 - Don't play, $01 - play once, $03 - Loop song		;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;


TrackLoop:
	db $00,$03,$03,$03,$03,$03,$03,$03,$03,$01,$01,$01,$01,$03,$03,$01	;\ 00-0F
	db $01,$01,$03,$01,$01,$01,$03,$01,$01,$03,$03,$01,$01,$01,$03,$03	;| 10-1F
	db $03,$03,$03,$03,$03,$01,$03,$01,$01,$01,$01,$01,$01,$01,$03,$03	;| 20-2F
	db $03,$01,$01,$03,$03,$01,$36,$37,$38,$39,$3A,$3B,$01,$01,$3E,$3F	;| 30-3F
	db $40,$41,$42,$43,$44,$45,$46,$47,$48,$49,$01				;/ 40-4A


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;	Switch to Overworld or Ending music bank									;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;


BankSwitch:
	LDA $0100|!Base2				;\
							;|
if !OWMusic						;|
	CMP #$0B					;| Load the current game mode, is it the current game mode 10 or lower?
	BCC Submap					;| if so, then we're on the overworld/title screen
endif							;| Title Screen < #$0B, Overworld < #$11
							;|
	CMP #$11					;|
	BCC Overworld					;/

	CMP #$18					;\ Otherwise, if the current game mode is 17 or less...
	BCC Level					;/ Then we're in a level.

	LDA $13C6|!Base2				;\
	CMP #$08					;| The cutscenes after boss battles bypass the Game Mode check, so this is needed to
	BCC Level					;/ manually point the songs played there to the Level bank

Credits:
	LDA $1DFB|!Base2				;\
	CLC						;| Set credits songs as tracks 27(in hex) and so on
	ADC #$1E					;/
	BRA Plus

Overworld:

if !OWMusic
	LDA $1BA0|!Base2				;\ Fix OW Music patch bug which precludes the "Valley Of Bowser appears" song from playing
	BNE Submap					;/
	LDA $0DB3|!Base2				;\ Mario or Luigi?
	BNE Weegee					;/
	LDX $1F11|!Base2				;\ If Mario is on Main map, don't branch
	BNE Submap					;/
	JSR GetMusicIndexM				; Get Mario position on Overworld
	BRA OWMusic

Weegee:
	LDX $1F12|!Base2				;\ If Luigi is on Main map, don't branch
	BNE Submap					;/
	JSR GetMusicIndexL				; Get Luigi position on Overworld

OWMusic:
	LDA.l MusicTBL,x				;\ Load music index from table and store it to free RAM, this will enable the routine in the
	STA !FreeRAM4					;/ very beginning of the this MSU-1 patch
	BRA Plus

Submap:
	STZ !FreeRAM4
endif

	LDA $1DFB|!Base2				;\
	CLC						;| Set Overworld songs as tracks 1E and so on
	ADC #$1D					;/

Plus:
	JMP Continue2					; Forward the modified song value

Level:
	JMP Continue1

if !OWMusic
GetMusicIndexM:
	LDA $1F21|!Base2
	ASL #3
	AND #$F0
	PHA
	LDA $1F1F|!Base2
	LSR
	ORA $01,s
	STA $01,s
	PLX
	RTS

GetMusicIndexL:
	LDA $1F25|!Base2
	ASL #3
	AND #$F0
	PHA
	LDA $1F23|!Base2
	LSR
	ORA $01,s
	STA $01,s
	PLX
	RTS


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;	Set multiple songs for Overworld. Each byte corresponds to a 32x32 block on the main map.			;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;


MusicTBL:
db $1F,$1F,$1F,$1F,$1F,$1F,$1F,$1F,$1F,$1F,$1F,$1F,$1F,$1F,$1F,$1F
db $1F,$1F,$1F,$1F,$1F,$1F,$1F,$1F,$1F,$2F,$2F,$2F,$33,$33,$33,$33
db $1F,$1F,$1F,$1F,$1F,$1F,$1F,$1F,$1F,$2F,$2F,$33,$33,$33,$33,$33
db $1F,$1F,$1F,$1F,$1F,$1F,$1F,$1F,$2F,$2F,$2F,$33,$33,$33,$33,$33
db $1F,$1F,$1F,$1F,$1F,$1F,$1F,$1F,$1F,$30,$30,$1F,$1F,$33,$33,$33
db $1F,$1F,$1F,$1F,$1F,$1F,$1F,$1F,$1F,$30,$30,$1F,$1F,$33,$33,$33
db $1F,$1F,$1F,$1F,$1F,$1F,$1F,$1F,$1F,$30,$30,$1F,$1F,$33,$33,$33
db $1F,$1F,$1F,$1F,$1F,$1F,$1F,$1F,$30,$30,$34,$34,$34,$34,$34,$34
db $1F,$1F,$1F,$1F,$1F,$1F,$1F,$1F,$30,$30,$34,$34,$34,$34,$34,$34
db $1F,$1F,$1F,$1F,$1F,$1F,$34,$34,$30,$30,$34,$34,$34,$34,$34,$34
db $1F,$1F,$1F,$1F,$1F,$34,$34,$34,$34,$34,$34,$34,$34,$34,$34,$34
db $1F,$1F,$1F,$1F,$1F,$34,$34,$34,$34,$34,$34,$34,$34,$1F,$1F,$1F
db $1F,$1F,$1F,$1F,$1F,$34,$34,$34,$34,$34,$34,$34,$34,$1F,$1F,$1F
db $1F,$1F,$1F,$1F,$1F,$34,$34,$34,$34,$34,$34,$34,$1F,$1F,$1F,$1F
db $1F,$1F,$1F,$1F,$1F,$34,$34,$34,$34,$34,$34,$34,$1F,$1F,$1F,$1F
db $1F,$1F,$1F,$1F,$1F,$1F,$1F,$1F,$34,$34,$34,$34,$1F,$1F,$1F,$1F
endif


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;	Pause MSU-1 flag												;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;


Paused:
	BEQ $02						;\ Restore previous routine
	LDY #$11					;/
	PHP
	SEP #$30
	PHA

	%CheckNoMSU(End)

	CPY #$11
	BNE NotPause
	LDA #!Pausebright
	STA $0DAE|!Base2
	LDA #!Volume					;\ Set MSU volume to the fade register
	STA !FreeRAM3					;/
	LDA #$80					;\ Activate THEFade routine
	STA $1DFB|!Base2				;/
	BRA End

NotPause:
	CPY #$12
	BNE End
	LDA #!Unpausebright
	STA $0DAE|!Base2
	LDA !FreeRAM2					;\ Rescue MSU audio control from free Ram and store it back to $2007
	STA $2007					;/
	LDA #!Volume					;\ Restore audio volume
	STA $2006					;/
	STZ !FreeRAM3

End:
	PLA
	PLP
	RTL


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;	Fix MSU-1 in fade screen											;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;


FadeOut:
	PHP
	SEP #$30
	PHA

	%CheckNoMSU(Fix)
	LDA $1DEF					;\
	CMP #$01					;| Check if song has the looping set or not
	BEQ Jingle					;/
	LDA #!Volume					;\ Set MSU volume to the fade register
	STA !FreeRAM3					;/

Screen:
	LDA !FreeRAM3					;\
	BEQ Fix						;|
	DEC A						;| Bull
	STA !FreeRAM3					;|
	STA $2006					;/
	BRA Screen

Jingle:
	LDA $2000					;\
	AND #$10					;| Lock the game in a black screen until the music stops playing, this is useful
	BNE Jingle					;| because prevents some songs from ending abruptly
	BEQ Fixed					;/

Fix:
	STZ !FreeRAM3					;\
							;|
if !OWMusic						;|
	STZ !FreeRAM4					;| Clear more flags and stop MSU-1 audio
endif							;|
							;|
	STZ $2007					;/

Fixed:
	PLA
	PLP
	LDA #$B1					;\ Restored native code of the music upload routine
	STA $0000|!Base1				;/
	RTL


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;	Fade out music													;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;


FadeMusic:
	PHP
	SEP #$30
	PHA

	%CheckNoMSU(GoEnd)

	LDA !FreeRAM3					;\
	BNE GoOn					;| Check if Fade Out flag is set, if not, then set it with the MSU-1 volume
	LDA #!Volume					;|
	STA !FreeRAM3					;/

GoOn:
	PLA
	PLP
	LDA #$80					;\
	STA $2142					;| Mute SPC registers
	STA $1DFB|!Base2				;|
	STA $1DFF|!Base2				;/
	RTL

GoEnd:
	PLA
	PLP
	LDA #$80					;\ Restored native code
	STA $1DFB|!Base2				;/
	RTL